@@ -29,8 +29,8 @@ typedef struct _AppChooserDialog AppChooserDialog;
 typedef struct _AppChooserDialogClass AppChooserDialogClass;
 
 AppChooserDialog * app_chooser_dialog_new (const char **app_ids,
-                                           const char *default_id,
-                                           const char *cancel_label,
-                                           const char *accept_label,
-                                           const char *title,
-                                           const char *heading);
+                                           const char  *default_id,
+                                           const char  *content_type,
+                                           const char  *filename);
+
+GAppInfo *app_chooser_dialog_get_info (AppChooserDialog *dialog);
